@@ -1,7 +1,7 @@
 /*
    Project: DataBasin
 
-   Copyright (C) 2009-2015 Free Software Foundation
+   Copyright (C) 2009-2017 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -25,20 +25,22 @@
 #import "DBCSVWriter.h"
 #import "DBSObject.h"
 #import "DBLoggerProtocol.h"
+#import "DBSFTypeWrappers.h"
+
+#import <WebServices/GWSConstants.h>
+
+NSString *DBFileFormatCSV = @"CSV";
 
 @implementation DBCSVWriter
 
 - (id)initWithHandle:(NSFileHandle *)fileHandle
 {
-  if ((self = [super init]))
+  if ((self = [super initWithHandle:fileHandle]))
     {
-      file = fileHandle;
       isQualified = YES;
       qualifier = @"\"";
       separator = @",";
       newLine = @"\n";
-      fieldNames = nil;
-      writeOrdered = NO;
       lineBreakHandling = DBCSVLineBreakNoChange;
       [self setStringEncoding: NSUTF8StringEncoding];
    }
@@ -49,23 +51,11 @@
 {
   [qualifier release];
   [separator release];
-  [fieldNames release];
   [super dealloc];
 }
 
-- (void)setWriteFieldsOrdered:(BOOL)flag
+- (void)setFileFormat:(NSString *)f
 {
-  writeOrdered = flag;
-}
-
-- (BOOL)writeFieldsOrdered
-{
-  return writeOrdered;
-}
-
-- (void)setLogger:(id<DBLoggerProtocol>)l
-{
-  logger = l;
 }
 
 - (void)setIsQualified: (BOOL)flag
@@ -101,28 +91,6 @@
   lineBreakHandling = handling;
 }
 
-- (void)setStringEncoding: (NSStringEncoding) enc
-{
-  NSData *tempData;
-
-  encoding = enc;
-  bomLength = 0;
-
-  /* BOM heuristics */
-  tempData = [@" "dataUsingEncoding: encoding];
-#if defined(__APPLE__) && (MAC_OS_X_VERSION_MAX_ALLOWED <= MAC_OS_X_VERSION_10_4)
-  NSString *blankString;
-    
-  blankString = [[[NSString alloc] initWithBytes: [tempData bytes] length: [tempData length] encoding: encoding] autorelease];
-  NSLog(@"blank string: %@", blankString);
-  tempData = [tempData subdataWithRange: NSMakeRange(0, [tempData length] - [blankString length])];
-#else
-  tempData = [tempData subdataWithRange: NSMakeRange(0, [tempData length] - [@" " lengthOfBytesUsingEncoding: encoding])];
-#endif
-  bomLength = [tempData length];
-
-  NSLog(@"bom length: %u", bomLength);
-}
 
 - (NSString *)formatScalarObject:(id)value
 {
@@ -172,8 +140,31 @@
 	  res = value;
 	}
     }
+  else if ([value isKindOfClass: [DBSFDataType class]])
+    {
+      NSLog(@"SF DataTYpe class: %@", [value class]);
+
+      if (isQualified)
+	{
+	  NSMutableString *s;
+	  NSString *strValue;
+
+	  strValue = [value stringValue];
+	  s = [[NSMutableString alloc] initWithCapacity: [strValue length]+2];
+	  [s appendString: qualifier];
+	  [s appendString: strValue];
+	  [s appendString: qualifier];
+	  res = [NSString stringWithString: s];
+	  [s release];
+	}
+      else
+	{
+	  res = [value stringValue];
+	}
+    }
   else if ([value isKindOfClass: [NSNumber class]])
     {
+      NSLog(@"number class: %@", [value class]);
       // FIXME: this is locale sensitive?
       // FIXME2: maybe give the option to quote also numbers
       if (isQualified)
@@ -201,160 +192,7 @@
   return res;
 }
 
-- (void)formatComplexObject:(NSMutableDictionary *)d withRoot:(NSString *)root inDict:(NSMutableDictionary *)dict inOrder:(NSMutableArray *)order
-{
-  NSMutableArray  *keys;
-  unsigned i;
-  NSString *extendedFieldName;
 
-  if (!d)
-    return;
-
-  keys = [NSMutableArray arrayWithArray:[d allKeys]];
-  [keys removeObject:@"GWSCoderOrder"];
-  
-  /* remove some fields which get added automatically by salesforce even if not asked for */
-  [keys removeObject:@"type"];
-  
-  /* remove Id only if it is null, else an array of two populated Id is returned by SF */
-  if (![[d objectForKey:@"Id"] isKindOfClass: [NSArray class]])
-    [keys removeObject:@"Id"];
-
-  //[logger log: LogDebug :@"[DBCSVWriter formatComplexObject] clean dictionary %@:\n", d];
-  //NSLog(@"[DBCSVWriter formatComplexObject] clean dictionary %@\n", d);
-
-  for (i = 0; i < [keys count]; i++)
-    {
-      id obj;
-      NSString *key;
-      
-      key = [keys objectAtIndex: i];
-      obj = [d objectForKey: key];
-      if ([key isEqualToString:@"Id"])
-	obj = [obj objectAtIndex: 0];
-      
-      if ([obj isKindOfClass: [NSDictionary class]])
-        {
-          NSMutableString *s;
-
-          if (root)
-            s = [NSMutableString stringWithString:root];
-          else
-            s = [NSMutableString stringWithString:@""];
-
-          if (root)
-            [s appendString:@"."];
-          [s appendString:key];
-
-          //NSLog(@"formatting complex object with root: %@", s);
-          [self formatComplexObject: obj withRoot:s inDict:dict inOrder:order];
-        }
-      else if ([obj isKindOfClass: [NSString class]] || [obj isKindOfClass: [NSNumber class]])
-        {
-	  NSMutableString *s;
-
-          if (root)
-            s = [NSMutableString stringWithString:root];
-          else
-            s = [NSMutableString stringWithString:@""];
-
-          if (root)
-            [s appendString:@"."];
-
-          [s appendString:key];
-
-          extendedFieldName = s;
-          //NSLog(@"formatting scalar object: %@ for key: %@", obj,extendedFieldName);
-          [dict setObject:obj forKey:extendedFieldName];
-          [order addObject:extendedFieldName];
-	}
-      else
-	NSLog(@"[DBCSVWriter formatComplexObject] unknown class of value: %@, object: %@", [obj class], obj);
-      
-    }
-}
-
-/*
-  This methods sets the internal field names for the header when using ordered object writeout.
-*/
-- (void)setFieldNames:(id)obj andWriteThem:(BOOL)flag
-{
-  NSArray *array;
-
-  [logger log: LogDebug :@"[DBCSVWriter setFieldNames] Object: %@:\n", obj];
-
-  /* if we have no data, we return */
-  if (obj == nil)
-    return;
-
-  /* if we have just a single object, we fake an array */
-  if([obj isKindOfClass: [NSArray class]])
-    array = obj;
-  else
-    array = [NSArray arrayWithObject: obj];
-
-  if ([array count] == 0)
-    return;
- 
-  if (fieldNames != array)
-    {
-      [fieldNames release];
-      fieldNames = array;
-      [array retain];
-    }
-
-  [logger log: LogDebug :@"[DBCSVWriter setFieldNames] Names: %@:\n", array];
-
-  /* if we write the header, fine, else we write at least the BOM */
-  if (flag == YES)
-    {
-      NSString *theLine;
-
-      theLine = [self formatOneLine:array forHeader:YES];
-      [file writeData: [theLine dataUsingEncoding: encoding]];
-    }
-  else
-    {
-      NSData *tempData;
-
-      tempData = [@" "dataUsingEncoding: encoding];
-      tempData = [tempData subdataWithRange: NSMakeRange(0, bomLength)];
-      [file writeData: tempData];
-    }
-}
-
-- (void)writeDataSet:(NSArray *)array
-{
-  NSUInteger i;
-  NSUInteger setCount;
-  NSAutoreleasePool *arp;
-
-  if (array == nil)
-    return;
-
-  arp = [[NSAutoreleasePool alloc] init];
-  setCount = [array count];
-  for (i = 0; i < setCount; i++)
-    {
-      NSString *oneLine;
-      NSData *data;
-      NSData *data2;
-      id o;
-
-//      NSLog(@"write data set");
-      o = [array objectAtIndex:i];
-      if ([o isKindOfClass: [DBSObject class]])
-	o = [NSArray  arrayWithObject: o];
-      oneLine = [self formatOneLine:o forHeader:NO];
-      data = [oneLine dataUsingEncoding: encoding];
-      if (bomLength > 0)
-	data2 = [NSData dataWithBytesNoCopy: (void *)[data bytes] length: [data length]-bomLength freeWhenDone: NO];
-      else
-	data2 = data;
-      [file writeData: data2];
-    }
-  [arp drain];
-}
 
 - (NSString *)formatOneLine:(id)data forHeader:(BOOL) headerFlag
 {
@@ -409,28 +247,40 @@
 	      value = [obj valueForField: key];
 	      //NSLog(@"key ---> %@ object %@", key, value);
 	      
-	      if ([value isKindOfClass: [NSString class]] ||[value isKindOfClass: [NSNumber class]] )
+	      if ([value isKindOfClass: [DBSFDataType class]])
+		{
+                  [dataDict setObject:[value stringValue] forKey:key];
+                  [keyOrder addObject:key];
+		}
+	      else if ([value isKindOfClass: [NSString class]] ||[value isKindOfClass: [NSNumber class]] )
 		{
                   [dataDict setObject:value forKey:key];
                   [keyOrder addObject:key];
 		}
-	      else if ([value isKindOfClass: [NSCalendarDate class]])
-		{
-		  // FIXME Date Handling could allow more options
-                  [dataDict setObject:[value description] forKey:key];
-                  [keyOrder addObject:key];
-		}
 	      else if ([value isKindOfClass: [NSDictionary class]])
 		{
-		  // NSLog(@"Dictionary");
 		  [self formatComplexObject:value withRoot:key inDict:dataDict inOrder:keyOrder];
 		}
+              else if ([value isKindOfClass: [NSArray class]])
+		{
+		  NSString *s;
+
+		  s = [NSString stringWithFormat:@"%lu subitems", (unsigned long)[(NSArray *)value count]];
+		  [dataDict setObject:s forKey:key];
+                  [keyOrder addObject:key];
+		}
 	      else
 		{
-		  NSLog(@"unknown class for object %@ of class %@", value, [value class]);
+		  NSLog(@"DBCSVWriter - formatOneLine - unknown class for object %@ of class %@ inside DBSObject", value, [value class]);
 		}
 	    }
 	}
+      else if ([obj isKindOfClass: [DBSFDataType class]])
+        {
+	  //NSLog(@"formatOneLine, we have directly a scalar object, NSString: %@", obj);
+          [dataDict setObject:obj forKey:obj];
+          [keyOrder addObject:obj];
+        }
       else if ([obj isKindOfClass: [NSString class]])
         {
 	  //NSLog(@"formatOneLine, we have directly a scalar object, NSString: %@", obj);
@@ -443,7 +293,10 @@
           [logger log: LogStandard :@"[DBCSVWriter formatOneLine] we have a NSNumber, unhandled %@:\n", obj];
 	}
       else
-        NSLog(@"unknown class of value: %@", [obj class]);
+	{
+	  [logger log: LogStandard :@"[DBCSVWriter formatOneLine] unknown class of value %@:\n", [obj class]];
+	  NSLog(@"[DBCSVWriter formatOneLine] unknown class of value: %@", [obj class]);
+	}
     }
 
   /* create the string */
