@@ -2,7 +2,7 @@
  Project: DataBasin
  DBSobject.m
  
- Copyright (C) 2010-2014 Free Software Foundation
+ Copyright (C) 2010-2017 Free Software Foundation
  
  Author: Riccardo Mottola
  
@@ -32,7 +32,7 @@
 @implementation DBSObject
 
 /** <p>Returns the corresponding 18-character case-insensitive
-    salesforce Id given the 15-character version <i>id15</i>
+    salesforce Id given the 15-character version <em>id15</em></p>
 */
 + (NSString *)idTo18: (NSString *) id15
 {
@@ -103,7 +103,7 @@
 
 - (NSString *)description
 {
-  return [NSString stringWithFormat: @"<DBSObject %@, %@, %@>", [self name], [self sfId], [self label]];
+  return [NSString stringWithFormat: @"<DBSObject %@, %@, %@, %@>", [self name], [self sfId], [self label], [self type]];
 }
 
 /** <p>returns the current salesforce id, in whichever format it is currently stored.</p>
@@ -160,7 +160,8 @@ converting it if necessary.</p>
 
 - (void)setObjectProperties: (NSDictionary *)properties
 {
-  [objectProperties release];
+  if (properties != objectProperties)
+    [objectProperties release];
   objectProperties = [[NSMutableDictionary dictionaryWithDictionary: properties] retain];
 }
 
@@ -215,7 +216,7 @@ converting it if necessary.</p>
   return fieldNames;
 }
 
-/** <p>Returns the value of field <i>field</i><p>
+/** <p>Returns the value of field <em>field</em></p>
 */
 - (id)valueForField: (NSString *)field
 {
@@ -246,8 +247,8 @@ converting it if necessary.</p>
 
 - (void)loadFieldValues
 {
-  unsigned int i;
-  unsigned int sizeCount;
+  NSUInteger i;
+  NSUInteger sizeCount;
   NSMutableArray *fieldsArray;
 
   if (dbs == nil)
@@ -260,7 +261,7 @@ converting it if necessary.</p>
       NSString *currField;
 
       currField = [fieldNames objectAtIndex:i];
-      if (sizeCount + [currField length] + 2 < MAX_SOQL_SIZE)
+      if (sizeCount + [currField length] + 2 < [dbs maxSOQLLength])
 	{
 	  [fieldsArray addObject: currField];
 	  sizeCount += [currField length] + 2;
@@ -282,7 +283,7 @@ converting it if necessary.</p>
   NSMutableString *statement;
   NSMutableArray *tempArray;
   DBSObject *tempObj;
-  int i;
+  NSUInteger i;
 
   if ([namesArray count] == 0)
     return;
@@ -324,4 +325,69 @@ converting it if necessary.</p>
     }
 }
 
+- (void)storeValuesForFields:(NSArray *)namesArray
+{
+  NSUInteger i;
+  DBSObject *tObj;
+  NSMutableArray *tArr;
+  NSMutableArray *results;
+  NSString *errorString;
+
+  if (!namesArray || [namesArray count] == 0)
+    return;
+
+  tObj = [[DBSObject alloc] init];
+  tArr = [[NSMutableArray alloc] initWithCapacity:1];
+  [tArr addObject:tObj];
+  [tObj release];
+
+  [tObj setObjectProperties:[self objectProperties]];
+  [tObj setValue:[self sfId] forField:@"Id"];
+  for (i = 0; i < [namesArray count]; i++)
+    {
+      NSString *fieldName;
+      NSString *fieldValue;
+    
+      fieldName = [namesArray objectAtIndex: i];
+      fieldValue = [self valueForField:fieldName];
+//      NSLog(@"%@ - %@", fieldName, fieldValue);
+      if (fieldValue)
+        [tObj setValue:fieldValue forField:fieldName];
+      else
+        NSLog(@"Error: trying to update field %@, but the field is null", fieldValue);
+    }
+  results = [dbs update :[tObj name] fromArray: tArr progressMonitor:nil];
+  
+  errorString = nil;
+  if (results && [results count] > 0)
+    {
+      NSUInteger j;
+    
+      for (j = 0; j < [results count]; j++)
+        {
+          NSDictionary *r;
+
+          r = [results objectAtIndex:j];
+          if ([[r objectForKey:@"success"] isEqualToString:@"false"])
+            {
+              if (errorString == nil)
+                errorString = [NSString stringWithString:[r objectForKey:@"message"]];
+              else
+                errorString = [[errorString stringByAppendingString:@", "] stringByAppendingString:[r objectForKey:@"message"]];
+            }
+        }
+    }
+  else
+    {
+      NSLog(@"Error: no valid results returned");
+    }
+
+  [tArr release];
+  if (errorString)
+    {
+      NSLog(@"Errors: %@", errorString);
+      [[NSException exceptionWithName:@"DBException" reason:errorString userInfo:nil] raise];
+    }
+}
+
 @end
