@@ -1,7 +1,7 @@
 /*
   Project: DataBasin
 
-  Copyright (C) 2008-2015 Free Software Foundation
+  Copyright (C) 2008-2018 Free Software Foundation
 
   Author: Riccardo Mottola
 
@@ -30,6 +30,7 @@
 #import "DBSoapCSV.h"
 #import "DBProgressProtocol.h"
 #import "DBLoggerProtocol.h"
+#import "DBCSVReader.h"
 
 /* since query identify would work in a big array giving memory issues, we split it up in this batch size */
 #define MAX_SIZE_OF_IDENTBATCH 20000
@@ -42,9 +43,9 @@
   logger = [db logger];
 }
 
-- (void)query :(NSString *)queryString queryAll:(BOOL)all toWriter:(DBCSVWriter *)writer progressMonitor:(id<DBProgressProtocol>)p
+- (void)query :(NSString *)queryString queryAll:(BOOL)all toWriter:(DBFileWriter *)writer progressMonitor:(id<DBProgressProtocol>)p
 {
-  int            batchSize;
+  NSUInteger     batchSize;
   NSArray        *fields;
   NSString       *qLoc;
   NSMutableArray *sObjects;
@@ -57,7 +58,10 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
   [dbSoap setDownBatchSize:[db downBatchSize]];
+  [dbSoap setEnableFieldTypesDescribeForQuery:[db enableFieldTypesDescribeForQuery]];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   fields = nil;
   if ([writer writeFieldsOrdered])
@@ -66,6 +70,8 @@
       [logger log: LogDebug :@"[DBSoapCSV query] query parsed fields: %@\n", fields];
     }
 
+  [writer writeStart];
+
   sObjects = [[NSMutableArray alloc] init];
 
   [p reset];
@@ -83,15 +89,10 @@
   batchSize = [sObjects count];
   if (batchSize > 0)
     {
-      if (fields != nil)
-        {
-          [writer setWriteFieldsOrdered:YES];
-          [writer setFieldNames: fields andWriteThem:YES];
-        }
+      if ([writer writeFieldsOrdered])
+        [writer setFieldNames: fields andWriteThem:YES];
       else
-        {
-          [writer setFieldNames: [sObjects objectAtIndex: 0] andWriteThem:YES];
-        }
+        [writer setFieldNames: [sObjects objectAtIndex: 0] andWriteThem:YES];
       [p setCurrentDescription:@"Writing"];
       [writer writeDataSet: sObjects];
       [p incrementCurrentValue:[sObjects count]];
@@ -120,8 +121,10 @@
       [p setCurrentDescription:@"Writing"];
       [writer writeDataSet: sObjects];
       [p incrementCurrentValue:[sObjects count]];
-      [arp drain];
+      [arp release];
     }
+
+  [writer writeEnd];
   [dbSoap release];
   [sObjects release];
   if ([p shouldStop])
@@ -132,20 +135,29 @@
 
 /**
    See DBSoap for informations about the batch size parameter.
+
+   The batch size parameter affects
  */
-- (void)queryIdentify :(NSString *)queryString queryAll:(BOOL)all fromReader:(DBCSVReader *)reader toWriter:(DBCSVWriter *)writer withBatchSize:(int)bSize progressMonitor:(id<DBProgressProtocol>)p
+- (void)queryIdentify :(NSString *)queryString queryAll:(BOOL)all fromReader:(DBCSVReader *)reader toWriter:(DBFileWriter *)writer withBatchSize:(int)bSize progressMonitor:(id<DBProgressProtocol>)p
 {
   NSArray        *inFieldNames;
   NSUInteger      inFieldCount;
   NSArray        *dataSet;
   NSMutableArray *identifierArray;
-  NSMutableArray *sObjects;
   NSUInteger     i;
   NSUInteger     batchSize;
   NSArray        *queryFields;
   GWSService     *serv;
   DBSoap         *dbSoap;
   BOOL           firstBatchIteration;
+  NSUInteger     identifyBatchSize;
+
+  /* if we identify through fields which are not keys and return a large number of results, the DBSoap identify,
+     We try to be smart and thus if Max is not selected as query batch size, then we sync the identify and the query
+     batch sizes */
+  identifyBatchSize = MAX_SIZE_OF_IDENTBATCH;
+  if (bSize > 0)
+    identifyBatchSize = bSize;
   
   /* we clone the soap instance and pass the session, so that the method can run in a separate thread */
   dbSoap = [[DBSoap alloc] init];
@@ -153,7 +165,10 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
   [dbSoap setDownBatchSize:2000];
+  [dbSoap setEnableFieldTypesDescribeForQuery:[db enableFieldTypesDescribeForQuery]];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   queryFields = nil;
   if ([writer writeFieldsOrdered])
@@ -163,7 +178,8 @@
     }
 
   [p reset];
-  
+
+  [writer writeStart];
   /* retrieve objects to create */
   
   /* first the fields */
@@ -186,35 +202,36 @@
     }
   
   [p setMaximumValue:[identifierArray count]];
-  sObjects = [[NSMutableArray alloc] init];
 
   [p setCurrentDescription:@"Identifying and querying."];
   [logger log: LogStandard :@"[DBSoapCSV queryIdentify] Identify through %@\n", inFieldNames];
 
-  firstBatchIteration = YES; /* we keep track of the first batch since we need to writ the header only once*/
+  firstBatchIteration = YES; /* we keep track of the first batch since we need to write the header only once*/
   while ([identifierArray count] > 0 && ![p shouldStop])
     {
       NSRange subArrayRange;
       NSArray *batchOfIdentifiers;
       NSAutoreleasePool *arp;
+      NSMutableArray *sObjects;
 
       arp = [[NSAutoreleasePool alloc] init];
       subArrayRange = NSMakeRange(0, [identifierArray count]);
-      if ([identifierArray count] > MAX_SIZE_OF_IDENTBATCH)
-        subArrayRange = NSMakeRange(0, MAX_SIZE_OF_IDENTBATCH);
+      if ([identifierArray count] > identifyBatchSize)
+        subArrayRange = NSMakeRange(0, identifyBatchSize);
       batchOfIdentifiers = [identifierArray subarrayWithRange:subArrayRange];
       [batchOfIdentifiers retain];
       [identifierArray removeObjectsInRange:subArrayRange];
 
+      sObjects = [[NSMutableArray alloc] init];
       NS_DURING
-        [db queryIdentify:queryString with:inFieldNames queryAll:all fromArray:batchOfIdentifiers toArray: sObjects withBatchSize:bSize progressMonitor: p];
+        [dbSoap queryIdentify:queryString with:inFieldNames queryAll:all fromArray:batchOfIdentifiers toArray: sObjects withBatchSize:bSize progressMonitor: p];
       NS_HANDLER
         [identifierArray release];
         [sObjects release];
         [dbSoap release];
         [localException raise];
         [batchOfIdentifiers release];
-        [arp drain];
+        [arp release];
       NS_ENDHANDLER
 
       [batchOfIdentifiers release];
@@ -224,9 +241,8 @@
         {
           if (firstBatchIteration)
             {
-              if (queryFields != nil)
+              if ([writer writeFieldsOrdered])
                 {
-                  [writer setWriteFieldsOrdered:YES];
                   [writer setFieldNames: queryFields andWriteThem:YES];
                 }
               else
@@ -236,12 +252,132 @@
               firstBatchIteration = NO;
             }
           [writer writeDataSet: sObjects];
-          [sObjects removeAllObjects];
         }
-      [arp drain];
+      [sObjects release];
+      [arp release];
     }
+  [writer writeEnd];
+  [dbSoap release];  
+  [identifierArray release];
+}
+
+- (void)retrieve :(NSString *)queryString fromReader:(DBCSVReader *)reader toWriter:(DBFileWriter *)writer withBatchSize:(int)bSize progressMonitor:(id<DBProgressProtocol>)p
+{
+  NSArray        *inFieldNames;
+  NSUInteger      inFieldCount;
+  NSArray        *dataSet;
+  NSMutableArray *identifierArray;
+  NSUInteger     i;
+  NSUInteger     batchSize;
+  NSArray        *queryFields;
+  GWSService     *serv;
+  DBSoap         *dbSoap;
+  BOOL           firstBatchIteration;
+  NSUInteger     retrieveBatchSize;
+
+  retrieveBatchSize = RETRIEVE_BATCH_SIZE;
+  if (bSize > 0 && bSize < RETRIEVE_BATCH_SIZE)
+    retrieveBatchSize = bSize;
+  
+  /* we clone the soap instance and pass the session, so that the method can run in a separate thread */
+  dbSoap = [[DBSoap alloc] init];
+  serv = [DBSoap gwserviceForDBSoap];
+  [dbSoap setSessionId:[db sessionId]];
+  [serv setURL:[db serverUrl]];  
+  [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
+  [dbSoap setEnableFieldTypesDescribeForQuery:[db enableFieldTypesDescribeForQuery]];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
+  
+  queryFields = nil;
+  if ([writer writeFieldsOrdered])
+    {
+      queryFields = [DBSoap fieldsByParsingQuery:queryString];
+      [logger log: LogDebug :@"[DBSoapCSV retrieve] query parsed fields: %@\n", queryFields];
+    }
+
+  [p reset];
+
+  [writer writeStart];
+  /* retrieve objects to create */
+  
+  /* first the fields */
+  [p setCurrentDescription:@"Loading data"];
+  inFieldNames = [reader fieldNames];
+  inFieldCount = [inFieldNames count];
+  dataSet = [reader readDataSet];
+  [logger log: LogDebug :@"[DBSoapCSV retrieve] field names: %@\n", inFieldNames];
+  
+  if (inFieldCount == 1)
+    {
+      identifierArray = [[NSMutableArray arrayWithCapacity: [dataSet count]] retain];
+      for (i = 0; i < [dataSet count]; i++)
+        [identifierArray addObject: [[dataSet objectAtIndex: i] objectAtIndex: 0]];
+    }
+  else
+    {
+      identifierArray = (NSMutableArray *)dataSet;
+      [identifierArray retain];
+    }
+  
+  [p setMaximumValue:[identifierArray count]];
+
+  [p setCurrentDescription:@"Retrieving."];
+
+  firstBatchIteration = YES; /* we keep track of the first batch since we need to write the header only once*/
+  while ([identifierArray count] > 0 && ![p shouldStop])
+    {
+      NSRange subArrayRange;
+      NSArray *batchOfIdentifiers;
+      NSAutoreleasePool *arp;
+      NSMutableArray *sObjects;
+      
+      arp = [[NSAutoreleasePool alloc] init];
+      subArrayRange = NSMakeRange(0, [identifierArray count]);
+      if ([identifierArray count] > retrieveBatchSize)
+        subArrayRange = NSMakeRange(0, retrieveBatchSize);
+      batchOfIdentifiers = [identifierArray subarrayWithRange:subArrayRange];
+      [batchOfIdentifiers retain];
+      NSLog(@"retrieve batch iteration, remaining count is: %lu, subarray size: %lu",  [identifierArray count], [batchOfIdentifiers count]);
+      [identifierArray removeObjectsInRange:subArrayRange];
+
+      sObjects = nil;
+      NS_DURING
+        sObjects = [dbSoap retrieveWithQuery:queryString andObjects:batchOfIdentifiers];
+      NS_HANDLER
+        [identifierArray release];
+        [dbSoap release];
+        [localException raise];
+        [batchOfIdentifiers release];
+        [arp release];
+      NS_ENDHANDLER
+
+      [sObjects retain];
+      [p incrementCurrentValue: [batchOfIdentifiers count]];
+      [batchOfIdentifiers release];
+      [p setCurrentDescription:@"Writing data"];
+      batchSize = [sObjects count];
+      if (batchSize > 0 )
+        {
+          if (firstBatchIteration)
+            {
+              if ([writer writeFieldsOrdered])
+                {
+                  [writer setFieldNames: queryFields andWriteThem:YES];
+                }
+              else
+                {
+                  [writer setFieldNames: [sObjects objectAtIndex: 0] andWriteThem:YES];
+                }
+              firstBatchIteration = NO;
+            }
+          [writer writeDataSet: sObjects];
+        }
+      [sObjects release];
+      [arp release];
+    }
+  [writer writeEnd];
   [dbSoap release];  
-  [sObjects release];
   [identifierArray release];
 }
 
@@ -263,6 +399,7 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   /* retrieve objects to create */
   
@@ -291,7 +428,7 @@
 
   resultArray = nil;
   NS_DURING
-    resultArray = [db create:objectName fromArray:sObjectsArray progressMonitor:p];
+    resultArray = [dbSoap create:objectName fromArray:sObjectsArray progressMonitor:p];
   NS_HANDLER
     [sObjectsArray release];
     [dbSoap release];
@@ -323,6 +460,8 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   /* retrieve objects to update */
   [p reset];
@@ -353,7 +492,7 @@
 
   resultArray = nil;
   NS_DURING
-    resultArray = [db update:objectName fromArray:sObjectsArray progressMonitor:p];
+    resultArray = [dbSoap update:objectName fromArray:sObjectsArray progressMonitor:p];
   NS_HANDLER
     [sObjectsArray release];
     [dbSoap release];
@@ -367,7 +506,7 @@
   return resultArray;
 }
 
-- (void)describeSObject: (NSString *)objectType toWriter:(DBCSVWriter *)writer
+- (void)describeSObject: (NSString *)objectType toWriter:(DBFileWriter *)writer
 {
   NSUInteger      i;
   NSUInteger     size;
@@ -385,9 +524,11 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   NS_DURING 
-    object = [db describeSObject: objectType];
+    object = [dbSoap describeSObject: objectType];
   NS_HANDLER
     [dbSoap release];
     [localException raise];
@@ -396,7 +537,8 @@
   
   fields = [object fieldNames];
   size = [fields count];
-  
+
+  [writer writeStart];
   
   keys = [[object propertiesOfField: [fields objectAtIndex: 0]] allKeys];
   [writer setFieldNames:[NSArray arrayWithArray:keys] andWriteThem:YES];
@@ -427,6 +569,7 @@
       [set addObject:values];
     }
   [writer writeDataSet:set];
+  [writer writeEnd];
   [set release];
   [dbSoap release];
 }
@@ -446,6 +589,8 @@
   [dbSoap setSessionId:[db sessionId]];
   [serv setURL:[db serverUrl]];  
   [dbSoap setService:serv];
+  [dbSoap setLogger:logger];
+  [dbSoap setSObjectDetailsDict:[db sObjectDetailsDict]];
   
   /* retrieve objects to delete */
   // FIXME perhaps this copy is useless
@@ -455,7 +600,7 @@
 
   resultArray = nil;
   NS_DURING
-    resultArray = [db delete:objectsArray progressMonitor:p];
+    resultArray = [dbSoap delete:objectsArray progressMonitor:p];
   NS_HANDLER
     [objectsArray release];
     [dbSoap release];
